@@ -14,23 +14,26 @@ if (!defined('MEDIAWIKI')) die();
 $wgAvailableRights[] = 'renameuser';
 $wgGroupPermissions['bureaucrat']['renameuser'] = true;
 
-$wgExtensionFunctions[] = 'wfSpecialRenameuser';
 $wgExtensionCredits['specialpage'][] = array(
 	'name' => 'Renameuser',
-	'author' => 'Ævar Arnfjörð Bjarmason',
+	'author' => 'Ævar Arnfjörð Bjarmason, Aaron Schulz',
 	'url' => 'http://www.mediawiki.org/wiki/Extension:Renameuser',
 	'description' => 'Rename a user (need \'\'renameuser\'\' right)',
+	'descriptionmsg' => 'renameuser-desc',
+	'svn-date' => '$LastChangedDate: 2008-05-20 03:07:11 -0400 (mar, 20 mai 2008) $',
+	'svn-revision' => '$Rev: 35076 $',
 );
 
 # Internationalisation file
-require_once( 'SpecialRenameuser.i18n.php' );
+$dir = dirname(__FILE__) . '/';
+$wgExtensionMessagesFiles['Renameuser'] = $dir . 'SpecialRenameuser.i18n.php';
 
 /**
  * The maximum number of edits a user can have and still be allowed renaming,
  * set it to 0 to disable the limit.
  */
-//define( 'RENAMEUSER_CONTRIBLIMIT', 6800 );
-define( 'RENAMEUSER_CONTRIBLIMIT', 200000 );
+define( 'RENAMEUSER_CONTRIBLIMIT', 2000000 );
+define( 'RENAMEUSER_CONTRIBJOB', 10000 );
 
 # Add a new log type
 global $wgLogTypes, $wgLogNames, $wgLogHeaders, $wgLogActions;
@@ -42,12 +45,5 @@ $wgLogActions['renameuser/renameuser'] = 'renameuserlogentry';
 $wgAutoloadClasses['SpecialRenameuser'] = dirname( __FILE__ ) . '/SpecialRenameuser_body.php';
 $wgAutoloadClasses['RenameUserJob'] = dirname(__FILE__) . '/RenameUserJob.php';
 $wgSpecialPages['Renameuser'] = 'SpecialRenameuser';
+$wgSpecialPageGroups['Renameuser'] = 'users';
 $wgJobClasses['renameUser'] = 'RenameUserJob';
-
-function wfSpecialRenameuser() {
-	# Add messages
-	global $wgMessageCache, $wgRenameuserMessages;
-	foreach( $wgRenameuserMessages as $key => $value ) {
-		$wgMessageCache->addMessages( $wgRenameuserMessages[$key], $key );
-	}
-}
\ No newline at end of file
